import random
import string

from datetime import datetime

from django.contrib.auth.models import User
from django.test import TestCase
from mock import patch

from locolanderweb.models import (
    STATUS_PENDING,
    MergeRequest,
    Project,
    RunLog,
)


class LocoLanderFactory(object):

    letterdigits = string.letters + string.digits

    def make_random_string(self, size=6, prefix=None):
        result = ''.join(random.choice(self.letterdigits)
                         for x in xrange(size))
        if prefix is not None:
            result = '%s-%s' % (prefix, result)
        return result

    def make_user(self, username=None, password='test', email=None):
        if username is None:
            username = self.make_random_string(prefix='user')
        if email is None:
            email = '%s@example.com' % username
        return User.objects.create_user(
            username=username, password=password, email=email)

    def make_project(self, name=None, user=None, url=None):
        if name is None:
            name = self.make_random_string(prefix='project-')
        if user is None:
            user = self.make_user()
        if url is None:
            url = 'http://foo.bar'
        return Project.objects.create(owner=user, url=url, name=name)

    def make_merge_request(
            self, project=None, status=STATUS_PENDING,
            source=None, target=None, commit_message=None,
            author=None, reviewers=None, date_created=None,
            date_completed=None):
        if project is None:
            project = self.make_project()
        if source is None:
            source = 'http://foo.source'
        if target is None:
            target = 'http://foo.target'
        if commit_message is None:
            commit_message = 'Foo bar commit message.'
        if author is None:
            author = 'Some Person <some@domain>'
        if reviewers is None:
            reviewers = ['mengano', 'fulano']
        if date_created is None:
            date_created = datetime.utcnow()
        return MergeRequest.objects.create(
            project=project, status=status, source=source, target=target,
            commit_message=commit_message, author=author, reviewers=reviewers,
            date_created=date_created, date_completed=date_completed)

    def make_run_log(self, merge_request=None, return_code=0, raw=None):
        if merge_request is None:
            merge_request = self.make_merge_request()
        if raw is None:
            raw = self.make_random_string(50)
        RunLog.objects.create(
            merge_request=merge_request, return_code=return_code, raw=raw)


class BaseTestCase(TestCase):

    password = '1234'
    factory = LocoLanderFactory()

    def setUp(self):
        super(BaseTestCase, self).setUp()
        p = patch('locolanderweb.models.get_service_from_url')
        self.get_service_from_url = p.start()
        self.addCleanup(p.stop)
