import cStringIO
import os
import pep8

from collections import defaultdict

from django.test import TestCase
from mock import patch
from pyflakes.scripts.pyflakes import checkPath

import locolander
import locolanderweb
import repos


class Pep8ConformanceTestCase(TestCase):

    packages = [locolander, locolanderweb, repos]
    exclude = ['migrations']

    def setUp(self):
        super(Pep8ConformanceTestCase, self).setUp()
        self.errors = {}
        self.pep8style = pep8.StyleGuide(
            counters=defaultdict(int),
            doctest='',
            exclude=self.exclude,
            filename=['*.py'],
            ignore=[],
            messages=self.errors,
            repeat=True,
            select=[],
            show_pep8=False,
            show_source=False,
            max_line_length=79,
            quiet=0,
            statistics=False,
            testsuite='',
            verbose=0
        )

    def test_pep8_conformance(self):
        for package in self.packages:
            self.pep8style.input_dir(os.path.dirname(package.__file__))
        self.assertEqual(self.pep8style.options.report.total_errors, 0)


class PyflakesAnalysisTestCase(TestCase):

    packages = [locolander, locolanderweb, repos]
    exclude_file = 'pyflakes-ignore.txt'

    def test_pyflakes_analysis(self):
        string_out = cStringIO.StringIO()
        with patch('sys.stdout', string_out):
            for package in self.packages:
                self._run_pyflakes_analysis(package)
        results = string_out.getvalue().splitlines()
        unexpected_errors = self._get_unexpected_errors(results)
        self.assertEqual(unexpected_errors, [],
                         '\n'.join(unexpected_errors))

    def _run_pyflakes_analysis(self, package):
        package_path = os.path.dirname(package.__file__)
        for dir_path, dir_names, file_names in os.walk(package_path):
            for file_name in file_names:
                if file_name.endswith('.py'):
                    file_path = os.path.join(dir_path, file_name)
                    checkPath(file_path)

    def _get_unexpected_errors(self, results):
        unexpected_errors = []
        expected_errors = self._parse_exclude_file()
        if not expected_errors:
            unexpected_errors = results
        else:
            for line in results:
                found = False
                for file_, error in expected_errors:
                    if file_ in line and error in line:
                        found = True
                        break
                if not found:
                    unexpected_errors.append(line)
        return unexpected_errors

    def _parse_exclude_file(self):
        excluded_errors = []
        if self.exclude_file is not None:
            exclude_file = os.path.join(os.path.dirname(__file__),
                                        self.exclude_file)
            with open(exclude_file) as file_:
                for line in file_:
                    if line.strip():
                        source_file, error = [data.strip() for data in
                                              line.split(':', 1)]
                        excluded_errors.append((source_file, error))
        return excluded_errors
