from django.core.urlresolvers import reverse
from pyquery import PyQuery

from locolanderweb.forms import ProjectForm
from locolanderweb.tests.test_models import BaseTestCase


class ProjectListTestCase(BaseTestCase):

    url = reverse('project-list')

    def setUp(self):
        super(ProjectListTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)

        self.factory.make_project(user=self.user)
        self.factory.make_project(user=self.user)
        self.factory.make_project(user=self.user)
        self.factory.make_project()  # project for another user

    def test_get(self):
        projects = self.user.project_set.all()
        assert projects.count() == 3
        response = self.client.get(self.url)

        self.assertContains(response, 'Your projects')
        html_projects = PyQuery(response.content).find('.project')
        self.assertEqual(len(html_projects), projects.count())
        for p in html_projects:
            name, url = tuple(i.text_content().strip()
                              for i in p.getchildren())
            self.assertEqual(
                self.user.project_set.filter(name=name, url=url).count(), 1)


class ProjectAddTestCase(BaseTestCase):

    url = reverse('project-add')

    def setUp(self):
        super(ProjectAddTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)
        self.data = dict(name='foo', url='lp:locolander')

    def test_get(self):
        assert self.user.project_set.all().count() == 0
        response = self.client.get(self.url)

        self.assertContains(response, ProjectForm(owner=self.user).as_table())
        self.assertEqual(self.user.project_set.all().count(), 0)

    def test_post(self):
        assert self.user.project_set.all().count() == 0
        response = self.client.post(self.url, data=self.data, follow=True)

        messages = [m.message for m in response.context['messages']]
        self.assertIn('Project foo added.', messages)


class ProjectDetailTestCase(BaseTestCase):

    def setUp(self):
        super(ProjectDetailTestCase, self).setUp()
        self.user = self.factory.make_user(password=self.password)
        assert self.client.login(username=self.user.username,
                                 password=self.password)

        self.project = self.factory.make_project(user=self.user)
        self.url = reverse('project-detail',
                           kwargs=dict(project_id=self.project.id))

    def test_get(self):
        response = self.client.get(self.url)

        self.assertContains(response, 'Project %s' % self.project.name)
